//+------------------------------------------------------------------+
//|                                          PC_SignalExpert_v1.mq4 |
//|                           Copyright  2007, TrendLaboratory Ltd. |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, TrendLaboratory Ltd."
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"


#include <stdlib.mqh>

//---- input parameters
extern string     Expert_Name    = "PC_SignalExpert_v1";

extern int        Magic          =   11111;
extern int        Slippage       =       3;

extern string     Main_data      = " Trade Volume & Trade Method";
extern double     Lots           =     0.1;
extern double     MaxOrders      =       1;
extern double     TakeProfit     =       0;    // Take Profit Value in pips 
extern int        StopLoss       =       0;    // Stop Loss value in pips 
extern int        TrailingStop   =       0;    // Trailing Stop value in pips 
extern double     BreakEven      =       0;    // Break-Even Value in pips
extern double     ProfitLock     =       0;    // Profit Lock in pips
extern int        ExitMode       =       0;    // Exit Mode: 0-off,1-by Signal
extern int        ReEntryMode    =       0;    // Re-Entry Mode: 0-off,1-on 
extern bool       SignalMail     =   false;    // E-mail switch 

extern string     Time_Inputs    = " Timing parameters ";
extern int        MainTimeFrame  =       0;    // Main TimeFrame in min 
extern int        StartHour      =       0;    // Start Hour of Trade Session 
extern int        StartMinute    =       0;    // Start Minute of Trade Session 
extern int        EndHour        =      23;    // End Hour of Trade Session
extern int        EndMinute      =       0;    // End Hour of Trade Session
extern int        CloseTimeMode  =       0;    // CloseTime mode: 0-off,1-on
extern int        CloseHour      =      24;    // Trade Close Hour   
extern int        FridayCloseHour=      22;    // Trade Close Hour in Friday
extern int        CloseMinute    =       0;    // Trade Close Minute   

extern string     Inputs      = " PriceChannel-Signal parameters ";
extern int        TimeFrame   =          0;    // TimeFrame in min
extern int        Risk        =          3;    // Risk factor (0..10) 
extern int        Length      =          9;    // Length (Period)

extern string     MM_inputs      = " MoneyManagement by L.Williams ";
extern bool       MM             =   false;    //  Switch
extern double     MMRisk         =    0.15;    // Risk Factor
extern double     MaxLoss        =    1000;    // Maximum Loss by 1 Lot

int      OrderBar=0;
double   Lotsi;
int      BEvent=0, TriesNum=5;
datetime CurrBar,PrevBar;

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
//---- 
   CurrBar = iTime(Symbol(),MainTimeFrame,0);
   PrevBar = iTime(Symbol(),MainTimeFrame,1);
   
//----
   return(0);
}

// ---- Scan Trades
int ScanTrades(int ord,int mode)
{   
   int total = OrdersTotal();
   int numords = 0;
   bool type = false; 
   int trd = 0;
   
   for(int cnt=0; cnt<total; cnt++) 
   {        
   OrderSelect(cnt, SELECT_BY_POS);            
   if ( ord != 0 )
   {
   if ( OrderType()==0 || OrderType()==2 || OrderType()==4 ) trd =  1;
   if ( OrderType()==1 || OrderType()==3 || OrderType()==5 ) trd =  2;      
   } else trd=0;
   
   if (mode == 0) type = OrderType()<=OP_SELLSTOP;
   if (mode == 1) type = OrderType()<=OP_SELL;   
   if (mode == 2) type = OrderType()>OP_SELL && OrderType()<=OP_SELLSTOP; 
   
   if(OrderSymbol() == Symbol() && type && trd==ord && OrderMagicNumber() == Magic)  
   numords++;
   }
   return(numords);
}  

int PCTrend(int mode)
{   
   int result = 0; 
   
   double OSL = 33-Risk;
   double OBL = 100 - OSL;
   
   double wpr  = 100 + iWPR(Symbol(),TimeFrame,Length,1);
   double pwpr = 100 + iWPR(Symbol(),TimeFrame,Length,2);
   
   double trend  = iCustom(NULL,TimeFrame,"PriceChannel_Signal_v1",Length,0,Risk,0,0,6,1); 
   double ptrend = iCustom(NULL,TimeFrame,"PriceChannel_Signal_v1",Length,0,Risk,0,0,6,2); 
      
   if ( mode == 1)
   {
   if (trend > 0) result =  1;
   if (trend < 0) result = -1; 
   }
   else
   if (mode == 2)
   {
   if ((ptrend < 0 && trend > 0) || (ReEntryMode>0 && pwpr < OBL && wpr > OBL && trend > 0)) result =  2;  
   if ((ptrend > 0 && trend < 0) || (ReEntryMode>0 && pwpr > OSL && wpr < OSL && trend < 0)) result = -2;
   }
         
   if(mode==0) result = 0;
   
   return(result); 
}

bool TimeCondition()
{
   bool result = false;
   
   datetime SessionStart = StrToTime(StartHour+":"+StartMinute);
   datetime SessionEnd   = StrToTime(EndHour+":"+EndMinute);
   
   if (StartHour < EndHour)
   result = TimeCurrent() >= SessionStart && TimeCurrent() < SessionEnd;      
   else
   if (StartHour > EndHour)
   result = (TimeCurrent() > SessionStart && TimeHour(TimeCurrent()) < 24)
          ||(TimeHour(TimeCurrent()) >= 0 && TimeCurrent() < SessionEnd);
   
   return(result);
}   

int TradeSignal()
{   
   int Trend = PCTrend(2);
     
   if (TimeCondition())
   {
      if ( Trend > 0 ) return( 1);
      if ( Trend < 0 ) return(-1);
   }
}

double MoneyManagement()
{
   double lot_min =MarketInfo(Symbol(),MODE_MINLOT);
   double lot_max =MarketInfo(Symbol(),MODE_MAXLOT);
   double lot_step=MarketInfo(Symbol(),MODE_LOTSTEP);
   double contract=MarketInfo(Symbol(),MODE_LOTSIZE);
   double vol;
//--- check data
   if(lot_min<0 || lot_max<=0.0 || lot_step<=0.0) 
   {
   Print("CalculateVolume: invalid MarketInfo() results [",lot_min,",",lot_max,",",lot_step,"]");
   return(0);
   }
   if(AccountLeverage()<=0)
   {
   Print("CalculateVolume: invalid AccountLeverage() [",AccountLeverage(),"]");
   return(0);
   }
//--- basic formula
   if ( MM )
   vol=NormalizeDouble(AccountFreeMargin()*MMRisk*AccountLeverage()/contract,2);
   else
   vol=Lots;
//--- check min, max and step
   vol=NormalizeDouble(vol/lot_step,0)*lot_step;
   if(vol<lot_min) vol=lot_min;
   if(vol>lot_max) vol=lot_max;
//---
   return(vol);
}   


void TrailStop()
{
   int    error;  
   bool   result=false;
   double Gain = 0;
    
   for (int cnt=0;cnt<OrdersTotal();cnt++)
   { 
   OrderSelect(cnt, SELECT_BY_POS);   
   int mode=OrderType();    
      if ( OrderSymbol()==Symbol() && OrderMagicNumber()==Magic) 
      {
         if (mode==OP_BUY) 
         {
			   if ( BreakEven > 0 && BEvent==0 )
			   {
			   Gain = (MarketInfo(Symbol(),MODE_BID) - OrderOpenPrice())/Point;
			      if( Gain >= BreakEven && OrderStopLoss()<=OrderOpenPrice()+ProfitLock*Point) 
			      {
			      double BuyStop = NormalizeDouble(OrderOpenPrice()+ProfitLock*Point,Digits);
			      BEvent=1;
			      }
			   }
			   else 			   
			   if( TrailingStop > 0) BuyStop = NormalizeDouble(MarketInfo(Symbol(),MODE_BID) - TrailingStop*Point,Digits);
			   
			   if( NormalizeDouble(OrderOpenPrice(),Digits)<= BuyStop) 
            {   
			      if ( BuyStop > NormalizeDouble(OrderStopLoss(),Digits) || OrderStopLoss() == 0) 
			      {
			         for(int k = 0 ; k < TriesNum; k++)
                  {
                  result = OrderModify(OrderTicket(),OrderOpenPrice(),
			                              BuyStop,
			                              OrderTakeProfit(),0,Lime);
                  error=GetLastError();
                     if(error==0) break;
                     else {Sleep(5000); RefreshRates(); continue;}
                  }   		 
               }            
            }
         }   
// - SELL Orders          
         if (mode==OP_SELL)
         {
            if ( BreakEven > 0 && BEvent==0)
			   {
			   Gain = (OrderOpenPrice()-MarketInfo(Symbol(),MODE_ASK))/Point;
			   
			      if( Gain >= BreakEven && (OrderStopLoss()>=OrderOpenPrice()-ProfitLock*Point || OrderStopLoss()== 0)) 
			      {
			      double SellStop = NormalizeDouble(OrderOpenPrice()-1*Point,Digits);
			      BEvent=-1;
			      }
			   }
			   else 
			   if( TrailingStop > 0) SellStop = NormalizeDouble(MarketInfo(Symbol(),MODE_ASK) + TrailingStop*Point,Digits);   
            
            if(NormalizeDouble(OrderOpenPrice(),Digits) >= SellStop && SellStop>0) 
            {
               if( SellStop < NormalizeDouble(OrderStopLoss(),Digits) || OrderStopLoss() == 0) 
               {
                  for( k = 0 ; k < TriesNum; k++)
                  {
                  result = OrderModify(OrderTicket(),OrderOpenPrice(),
			                              SellStop,
			                              OrderTakeProfit(),0,Orange);
                  error=GetLastError();
                     if(error==0) break;
                     else {Sleep(5000); RefreshRates(); continue;}
                  }
               }   
   			}	    
         }
      }
   }     
}


// ---- Open Sell Orders
void SellOrdOpen()
{		     
   double SellPrice = Bid;
   double StopPrice = Bid;
   int    Mode = OP_SELL;
        	  
   if (StopLoss > 0) 
   double SellStop  = SellPrice + StopLoss*Point;
   else SellStop = 0;
	
   if (TakeProfit  > 0) double SellProfit = SellPrice - TakeProfit*Point; 
   else SellProfit=0;
   
	int ticket = OrderSend( Symbol(),Mode,MoneyManagement(),
	                        NormalizeDouble(SellPrice , Digits),
	                        Slippage,
	                        NormalizeDouble(SellStop  , Digits),
	                        NormalizeDouble(SellProfit, Digits),
	                        Expert_Name+" SELL",Magic,0,Red);
   
   if(ticket > 0) 
   {
      if (OrderSelect(ticket, SELECT_BY_TICKET, MODE_TRADES)) 
      {
		//OrderBar =iBars(OrderSymbol(),BT_TimeFrame);
		Print("SELL order opened : ", OrderOpenPrice());
      BEvent=0;
      if (SignalMail) SendMail("[Signal Alert]", "[" + Symbol() + "] " + DoubleToStr(Bid, Digits) + " Open Sell");
		}
   }
	else 	
   if(GetLastError()>0)
   Print("SELL: OrderSend failed with error #",GetLastError());
   
   //return(0);
}
// ---- Open Buy Orders
void BuyOrdOpen()
{		     
   double BuyPrice  = Ask;
   double StopPrice = Ask;
   int    Mode      = OP_BUY;
      
   if (StopLoss > 0) 
   double BuyStop  = BuyPrice - StopLoss*Point;
   else BuyStop = 0;
   
   if (TakeProfit  > 0) double BuyProfit= BuyPrice + TakeProfit*Point;  
   else BuyProfit=0;  
		 
	int ticket = OrderSend(Symbol(),Mode, MoneyManagement(),
	                   NormalizeDouble(BuyPrice , Digits),
	                   Slippage,
	                   NormalizeDouble(BuyStop  , Digits), 
	                   NormalizeDouble(BuyProfit, Digits),
	                   Expert_Name+" BUY",Magic,0,Blue);
      
   if(ticket > 0) 
   {
      if (OrderSelect(ticket, SELECT_BY_TICKET, MODE_TRADES)) 
      {
		//OrderBar =iBars(OrderSymbol(),BT_TimeFrame); 
		Print("BUY order opened : ", OrderOpenPrice());
      BEvent=0;
      if (SignalMail) SendMail("[Signal Alert]", "[" + Symbol() + "] " + DoubleToStr(Ask, Digits) + " Open Buy");
		}
   }
	else 	
   if(GetLastError()>0)      
   Print("BUY : OrderSend failed with error #",GetLastError());

   //return(0);
} 

void CloseOrder(int mode)  
{
   bool result=false; 
   int  total=OrdersTotal();
   
   for (int i=0;i<total;i++)  
   {
   if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
   if (OrderMagicNumber() != Magic || OrderSymbol() != Symbol()) continue; 
      
   if ((mode == 0 || mode ==1) && OrderType()==OP_BUY ) {result=CloseAtMarket(OrderTicket(),OrderLots(),Aqua);break;}
   if ((mode == 0 || mode ==2) && OrderType()==OP_SELL) {result=CloseAtMarket(OrderTicket(),OrderLots(),Pink);break;}
   }
}


bool CloseAtMarket(int ticket,double lot,color clr) 
{
   bool result = false; 
   int  ntr;
      
   int tries=0;
   while (!result && tries < TriesNum) 
   {
      //ntr=0; 
      //while (ntr<5 && !IsTradeAllowed()) { ntr++; Sleep(5000); }
      RefreshRates();
      result=OrderClose(ticket,lot,OrderClosePrice(),Slippage,clr);
      
      tries++;
   }
   if (!result) Print("Error closing order : ",ErrorDescription(GetLastError()));
   return(result);
}

//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//---- 
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{
   if(iBars(Symbol(),MainTimeFrame)< 100 || IsTradeAllowed()==false) return(0); 
      
   if(AccountFreeMargin()< 1000*MoneyManagement())
   {
   Print("We have no money. Free Margin = ", AccountFreeMargin());
   return(0);  
   }
   CurrBar = iTime(Symbol(),MainTimeFrame,0);
   
   if( DayOfWeek()!=5) datetime TimeClose = StrToTime(CloseHour+":"+CloseMinute); 
   else TimeClose = StrToTime(FridayCloseHour+":"+CloseMinute); 
   
   int Signal = 0;
   int Exit = 0;
   if (CurrBar != PrevBar)
   {
   Signal = TradeSignal();
   Exit = PCTrend(1);
   }
   
   if(ScanTrades(0,1) > 0)
   {
      if(ExitMode > 0 && Exit != 0)
      {
         if (Exit > 0 ) while(ScanTrades(2,1)>0) CloseOrder(2);
         else
         if (Exit < 0 ) while(ScanTrades(1,1)>0) CloseOrder(1);
      }
   
   if(BreakEven>0 || TrailingStop>0) TrailStop();
   if (CloseTimeMode > 0 && TimeCurrent() >= TimeClose) CloseOrder(0);
   }  
      
   if(ScanTrades(0,1) < MaxOrders && Signal != 0)
   {
      if (Signal > 0) BuyOrdOpen();  
      else
      if (Signal < 0) SellOrdOpen();
   }
  
   PrevBar = CurrBar;
   
//----
   return(0);
} //start()
//+------------------------------------------------------------------+